<?php
/**
 * technerd functions and definitions
 *
 * Set up the theme and provides some helper functions, which are used in the
 * theme as custom template tags. Others are attached to action and filter
 * hooks in WordPress to change core functionality.
 *
 * When using a child theme you can override certain functions (those wrapped
 * in a function_exists() call) by defining them first in your child theme's
 * functions.php file. The child theme's functions.php file is included before
 * the parent theme's file, so the child theme functions would be used.
 *
 * @link https://codex.wordpress.org/Theme_Development
 * @link https://codex.wordpress.org/Child_Themes
 *
 * Functions that are not pluggable (not wrapped in function_exists()) are
 * instead attached to a filter or action hook.
 *
 * For more information on hooks, actions, and filters,
 * {@link https://codex.wordpress.org/Plugin_API}
 *
 * @package technerd
 * @since technerd 1.0
 */

/**
 * Set the content width based on the theme's design and stylesheet.
 *
 * @since technerd 1.0
 */
if ( ! isset( $content_width ) ) {
	$content_width = 660;
}

if ( ! function_exists( 'technerd_setup' ) ) :
/**
 * Sets up theme defaults and registers support for various WordPress features.
 *
 * Note that this function is hooked into the after_setup_theme hook, which
 * runs before the init hook. The init hook is too late for some features, such
 * as indicating support for post thumbnails.
 *
 * @since technerd 1.0
 */
function technerd_setup() {

	/*
	 * Make theme available for translation.
	 * Translations can be filed in the /languages/ directory.
	 */
	load_theme_textdomain( 'technerd', get_template_directory() . '/languages' );

	// Add default posts and comments RSS feed links to head.
	add_theme_support( 'automatic-feed-links' );

	/*
	 * Let WordPress manage the document title.
	 * By adding theme support, we declare that this theme does not use a
	 * hard-coded <title> tag in the document head, and expect WordPress to
	 * provide it for us.
	 */
	add_theme_support( 'title-tag' );

	/*
	 * Enable support for Post Thumbnails on posts and pages.
	 *
	 * See: https://codex.wordpress.org/Function_Reference/add_theme_support#Post_Thumbnails
	 */
	add_theme_support( 'post-thumbnails' );

	// Add custom logo support
	add_theme_support( 'custom-logo' );

	// Add new image sizes
	add_image_size( 'featured-image', 795, 9999 );

	// This theme uses wp_nav_menu() in one locations.
	register_nav_menus( array(
		'primary' => __( 'Primary Menu', 'technerd' ),
	) );

	/*
	 * Switch default core markup for search form, comment form, and comments
	 * to output valid HTML5.
	 */
	add_theme_support( 'html5', array(
		'search-form', 'comment-form', 'comment-list', 'gallery', 'caption'
	) );
}
endif; // technerd_setup
add_action( 'after_setup_theme', 'technerd_setup' );

/**
 * Register widget area.
 *
 * @since technerd 1.0
 *
 * @link https://codex.wordpress.org/Function_Reference/register_sidebar
 */
function technerd_widgets_init() {
	register_sidebar( array(
		'name'          => __( 'Right Sidebar', 'technerd' ),
		'id'            => 'sidebar-right',
		'description'   => __( 'Add widgets here to appear in your sidebar.', 'technerd' ),
		'before_widget' => '<aside id="%1$s" class="widget %2$s">',
		'after_widget'  => '</aside>',
		'before_title'  => '<h3 class="widget-title">',
		'after_title'   => '</h3>',
	) );
	register_sidebar( array(
		'name'          => __( 'Footer', 'technerd' ),
		'id'            => 'sidebar-footer',
		'description'   => __( 'Add widgets here to appear in your footer.', 'technerd' ),
		'before_widget' => '<aside id="%1$s" class="widget %2$s">',
		'after_widget'  => '</aside>',
		'before_title'  => '<h3 class="widget-title">',
		'after_title'   => '</h3>',
	) );
}
add_action( 'widgets_init', 'technerd_widgets_init' );

/**
 * JavaScript Detection.
 *
 * Adds a `js` class to the root `<html>` element when JavaScript is detected.
 *
 * @since technerd 1.0
 */
function technerd_javascript_detection() {
	echo "<script>(function(html){html.className = html.className.replace(/\bno-js\b/,'js')})(document.documentElement);</script>\n";
}
add_action( 'wp_head', 'technerd_javascript_detection', 0 );

/**
 * Enqueue scripts and styles.
 *
 * @since technerd 1.0
 */
function technerd_scripts() {
	
	// Scripts
	wp_enqueue_script( 'technerd-modernizr', get_template_directory_uri() . '/assets/js/modernizr-custom.js', array( 'jquery' ), '3.2.0', true );
	wp_enqueue_script( 'fitvids', get_template_directory_uri() . '/assets/js/jquery.fitvids.js', array( 'jquery' ), '1.0.3', true );
	wp_enqueue_script( 'flexslider', get_template_directory_uri() . '/assets/js/jquery.flexslider.js', array( 'jquery' ), '2.6.0', true );	
	wp_enqueue_script( 'technerd-technerd', get_template_directory_uri() . '/assets/js/technerd.js', array( 'jquery' ), '1.0.0', true );	

	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' );
	}

  // Stylesheets
  wp_enqueue_style( 'technerd-font-awesome-css', get_template_directory_uri() . '/assets/css/font-awesome.css', false, '4.6.3','all' );
  wp_enqueue_style( 'technerd-flexslider-css', get_template_directory_uri() . '/assets/css/flexslider.css', false, '2.6.0','all' );
	wp_enqueue_style( 'technerd-css', get_stylesheet_uri() );

}
add_action( 'wp_enqueue_scripts', 'technerd_scripts' );

/* Editor Style Sheet */

function technerd_add_editor_style() {
    add_editor_style( 'assets/css/custom-editor-style.css' );
}
add_action( 'after_setup_theme', 'technerd_add_editor_style' );

/**
 * Add Google Fonts
 */

function technerd_add_google_fonts() {
	wp_enqueue_style( 'technerd-google-fonts', 'http://fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,700italic,400,700,300|Libre+Baskerville:400,400italic,700bold|Limelight:400|Unica+One|Contrail+One', false ); 
}

add_action( 'wp_enqueue_scripts', 'technerd_add_google_fonts' );

/* 
 * Equal Sized Menu Items
 * @author Bill Erickson
 * @link http://www.billerickson.net/menu-item-equal-widths/
 */

function technerd_equal_sized_menu_items( $classes, $item, $args ) {
  
  // Only run on the header menu
  if( 'primary' !== $args->theme_location )
    return $classes;
  
  // Only run on top level menu items
  if( $item->menu_item_parent )
    return $classes;
  
  // Grab all menu items  
  $menu_items = wp_get_nav_menu_items( $args->menu->term_id );
  
  // Remove submenu items
  foreach( $menu_items as $i => $menu_item )
    if( $menu_item->menu_item_parent )
      unset( $menu_items[$i] );
  
  // Count all top level items, and add as a class
  $classes[] = 'count-' . count( $menu_items );
  return $classes;
}
add_filter( 'nav_menu_css_class', 'technerd_equal_sized_menu_items', 10, 3 );

/** 
 * Remove duplicate posts from main query if slider active
 */

function technerd_exclude_slider_posts( $query ) {

	$enabled = get_theme_mod( 'enable_slider' );

  if ( $enabled && $query->is_home() && $query->is_main_query() ) {
  	 $query->set( 'post__not_in', get_option( 'sticky_posts' ) );
  }
}
add_action( 'pre_get_posts', 'technerd_exclude_slider_posts' );

/**
 * Add Link to Excerpt
 *
 * @since technerd 1.0
 */

function technerd_excerpt_more( $more ) {
	return ' <a class="read-more-excerpt" href="' . get_permalink( get_the_ID() ) . '">' . __( '[...] Read More', 'technerd' ) . '</a>';
}
add_filter( 'excerpt_more', 'technerd_excerpt_more' );

/**
 * Custom template tags for this theme.
 *
 * @since technerd 1.0
 */
require get_template_directory() . '/inc/template-tags.php';

/**
 * Custom Options.
 *
 * @since technerd 1.0
 */
require get_template_directory() . '/inc/customizer.php';